/*
 * lingot, a musical instrument tuner.
 *
 * Copyright (C) 2004-2011  Ibán Cereijo Graña, Jairo Chapela Martínez.
 *
 * This file is part of lingot.
 *
 * lingot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * lingot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with lingot; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>

#include "lingot-defs.h"
#include "lingot-audio-portaudio.h"
#include "lingot-i18n.h"
#include "lingot-msg.h"
#include <string.h>

LingotAudioHandler* lingot_audio_portaudio_new(char* device, int sample_rate) {

	LingotAudioHandler* audio = NULL;

#	ifdef PORTAUDIO
	char error_message[1000];
	PaError err;
	const char* exception;
	int channels = 1;
	const PaStreamInfo* s_info;
	PaDeviceIndex dev_idx_default;
	const PaDeviceInfo* dev_info;

	audio = malloc(sizeof(LingotAudioHandler));
	audio->read_buffer = NULL;
	audio->audio_system = AUDIO_SYSTEM_PORTAUDIO;
	audio->read_buffer_size = 128;

	try {
		if ((err = Pa_Initialize()) != paNoError)
		{
			sprintf(error_message, "Pa_Initialize error: %s\n",
				Pa_GetErrorText(err));
			throw(error_message);
		}

		dev_idx_default = Pa_GetDefaultInputDevice();
		dev_info = Pa_GetDeviceInfo(dev_idx_default);
		strcpy(audio->device, dev_info->name);

		if ((err = Pa_OpenDefaultStream( &audio->portaudio_stream, channels, 0, paInt16, sample_rate,
				audio->read_buffer_size, NULL,
				NULL)) != paNoError)
		{
			sprintf(error_message, "Pa_OpenDefaultStream error: %s\n",
				Pa_GetErrorText(err));
			throw(error_message);
		}
		if ((err = Pa_StartStream(audio->portaudio_stream)) != paNoError)
		{
			sprintf(error_message, "Pa_StartStream error: %s\n",
				Pa_GetErrorText(err));
			throw(error_message);
		}

		s_info = Pa_GetStreamInfo(audio->portaudio_stream);
		audio->real_sample_rate = s_info->sampleRate;

		audio->read_buffer = malloc(channels * audio->read_buffer_size
				* Pa_GetSampleSize(paInt16));
		memset(audio->read_buffer, 0, audio->read_buffer_size
				* Pa_GetSampleSize(paInt16));

	} catch {
		if (audio->portaudio_stream != NULL)
			Pa_CloseStream(audio->portaudio_stream);
		free(audio);
		audio = NULL;
		lingot_msg_add_error(exception);
	}

#	else
	lingot_msg_add_error(
			_("The application has not been built with PORTAUDIO support"));
#	endif

	return audio;
}

void lingot_audio_portaudio_destroy(LingotAudioHandler* audio) {
#	ifdef PORTAUDIO
	if (audio != NULL) {
		Pa_CloseStream(audio->portaudio_stream);
		free(audio->read_buffer);
	}
#	endif
}

int lingot_audio_portaudio_read(LingotAudioHandler* audio) {
#	ifdef PORTAUDIO
	PaError err;
	int i;

	if (((err = Pa_ReadStream(audio->portaudio_stream, audio->read_buffer,
			audio->read_buffer_size)) != paNoError) && (err != paInputOverflowed))
	{
		char buff[100];
		sprintf(buff, _("Read from audio interface failed.\n%s."),
				Pa_GetErrorText(err));
		lingot_msg_add_error(buff);
		return -1;
	}

	// float point conversion
	for (i = 0; i < audio->read_buffer_size; i++) {
		audio->flt_read_buffer[i] = audio->read_buffer[i];
	}
#	endif

	return 0;
}

LingotAudioSystemProperties* lingot_audio_portaudio_get_audio_system_properties(
		audio_system_t audio_system) {

	LingotAudioSystemProperties* result =
			(LingotAudioSystemProperties*) malloc(1
					* sizeof(LingotAudioSystemProperties));

	// TODO
	result->forced_sample_rate = 0;
	result->n_devices = 0;
	result->devices = NULL;

	result->n_sample_rates = 5;
	result->sample_rates = malloc(result->n_sample_rates * sizeof(int));
	result->sample_rates[0] = 8000;
	result->sample_rates[1] = 11025;
	result->sample_rates[2] = 22050;
	result->sample_rates[3] = 44100;
	result->sample_rates[4] = 48000;

	return result;
}

